/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _sequence_
#define _sequence_

#include "wx/wx.h"
#include "wx/wfstream.h"

#include "irrXML/irrXML.h"

#include "Config.h"
#include "ptr_vector.h"
#include "GUI/GraphicalTrack.h"
#include "Midi/Track.h"

#include "Config.h"

#include <string>

namespace AriaMaestosa {
	
class ControllerEvent;

class GLPane;


class Sequence {
	
	DECLARE_LEAK_CHECK();
	
    int tempo;
    int beatResolution;
	
    float zoom; int zoom_percent;
	
    int x_scroll_in_pixels, y_scroll;
    int measureTopValue, measureBottomValue;
    //int measureWidth;
    
    bool scrollToPlaybackPosition;
    
    int reordering_newPosition; // used when reordering tracks, to hold the new position of the track being moved
	int reorderYScroll; // while reordering tracks, contains the vertical scrolling amount
	
    std::string copyright;
	
    int measureAmount;
	int currentTrack;
	
	ptr_vector<Track> tracks;
	
 public:   
    
    // this object is to be modified by MainFrame, to remember where to save this sequence
    wxString filepath;
    
    // these objects are to be modified by tracks
    int lastModifiedTrack;
    ptr_vector<Note> notesUndoMemory;
    ptr_vector<ControllerEvent> controllerUndoMemory;
	bool somethingDoneToUndo; // is undo currently possible?
    int x_scroll_upon_copying; // will store the horizontal scrolling when copying, and upon pasting behaviour will depend if x_scroll has changed since copy
	int notes_shift_when_no_scrolling; // if no scrolling is done, this value will be used to determine where to place notes
	
	// set this flag true to follow playback
	bool follow_playback;
	
    // ------------ read-only -------------
    bool importing; // set to true when importing - indicates the sequence will have frequent changes and not compute too much until it's over
    
    MainFrame* mainFrame;
    GLPane* glPane;

    int firstMeasure;
    int startTick;
    
    // dock
    int dockSize;
    ptr_vector<GraphicalTrack> dock;
    int dockHeight;
    
    wxString sequenceName;
    
    ptr_vector<ControllerEvent> tempoEvents;
    // ------------------------------------
    
    Sequence(GLPane* parent);
	~Sequence();
	
    void addToDock(GraphicalTrack* track);
    void removeFromDock(GraphicalTrack* track);
    
    void loadUndoMemory();
    
    void spacePressed();
    void renderTracks(int currentTick, int mousex, int mousey, int mousey_initial, int from_y);
    
    void reorderTracks(); // called when mouse is released after having dragged a track.
    
    void scrollNowToPlaybackPosition();
    
	// called repeatedly when mouse is held down
	void mouseHeldDown(int mousex_initial, int mousey_initial, int mousex_current, int mousey_current);
	
	// do we need to start a timer that will frequently send mouse held down events?
	bool areMouseHeldDownEventsNeeded();
	
	int getTrackAmount();
	int getCurrentTrackID();
	Track* getTrack(int ID);
	Track* getCurrentTrack();
	void setCurrentTrackID(int ID);
	void setCurrentTrack(Track* track);
	void addTrack();
    void deleteTrack();
	void deleteTrack(int ID);
	
    int getTotalHeight();
    void setTempo(int tempo);
    int getTempo();
    void setMeasureAmount(int measureAmount);
    int getMeasureAmount();
	
    int getZoomInPercent();
	float getZoom();
    void setZoom(int percent);
	
	void setXScrollInMidiTicks(int value);
    void setXScrollInPixels(int value);
    int getXScrollInMidiTicks();
    int getXScrollInPixels();
    
    void setYScroll(int value);
    int getYScroll();
    
    // called reacting to the user selecting "snap notes to grid" from the edit menu
    void snapNotesToGrid();
    
    void prepareEmptyTracksForLoading(int amount);
    
    void setFirstMeasure(int firstMeasureID);
    void setMeasureSig(int top, int bottom);
    int getMeasureTopValue();
    int getMeasureBottomValue();
    
    void setStartTick(int newValue);
    
    void setCopyright( char* copyright );
    std::string getCopyright();
    
    void scale(
               float factor,
               bool rel_first_note, bool rel_begin, // relative to what (only one must be true)
               bool affect_selection, bool affect_track, bool affect_song // scale what (only one must be true)
               );
    
    void enterPlayLoop();
    
    void copy();
    void paste();
    void pasteAtMouse();
    void selectAll();
    void selectNone();
    
    int ticksPerMeasure();
    int ticksPerBeat();
    void setTicksPerBeat(int res);
    
    // serialization
    void saveToFile(wxFileOutputStream& fileout);
    bool readFromFile(irr::io::IrrXMLReader* xml);
    
};

}
#endif
